using System;
using JetBrains.Annotations;
using UniRx;

namespace Magify
{
    public partial class MagifyManager
    {
        public static class Features
        {
            public static event Action OnFeaturesUpdated;
            private static IDisposable _onUpdateSubscription;
            private static IDisposable _onUpdateFeaturesInstance;

            public static bool IsCurrentFeaturesParsed
            {
                get
                {
                    ThrowIfMagifyIsNotReady(nameof(IsCurrentFeaturesParsed));
                    return MagifyPlatformAPI.Features.IsCurrentFeaturesParsed;
                }
            }

            /// <inheritdoc cref="ThrowIfMagifyIsNotReady"/>
            [NotNull]
            public static IStoredAppFeaturesCollection StoredAppFeatures
            {
                get
                {
                    ThrowIfMagifyIsNotReady(nameof(StoredAppFeatures));
                    return MagifyPlatformAPI.StoredAppFeatures;
                }
            }

            internal static void Initialize()
            {
                _onUpdateFeaturesInstance = MagifyPlatformAPI.ObserveEveryValueChanged(api => api?.Features)
                    .Subscribe(_ => reSubscribe());

                void reSubscribe()
                {
                    _onUpdateSubscription?.Dispose();
                    _onUpdateSubscription = Observable
                        .FromEvent(h => MagifyPlatformAPI.Features.OnUpdate += h, h => MagifyPlatformAPI.Features.OnUpdate -= h)
                        .Subscribe(_ => OnFeaturesUpdated?.Invoke());
                }
            }

            /// <summary>
            /// For correct events work it require to call Initialize() again (with actual MagifyPlatformAPI)
            /// </summary>
            internal static void Reset()
            {
                OnFeaturesUpdated = null;
                _onUpdateFeaturesInstance?.Dispose();
                _onUpdateFeaturesInstance = null;
            }

            public static bool TryGetBool(string featureName, out bool result)
            {
                ThrowIfMagifyIsNotReady(nameof(TryGetBool));
                return MagifyPlatformAPI.Features.TryGetBool(featureName, out result);
            }

            public static bool GetBool(string featureName)
            {
                ThrowIfMagifyIsNotReady(nameof(GetBool));
                return MagifyPlatformAPI.Features.GetBool(featureName);
            }

            public static bool TryGetNumber(string featureName, out double result)
            {
                ThrowIfMagifyIsNotReady(nameof(TryGetNumber));
                return MagifyPlatformAPI.Features.TryGetNumber(featureName, out result);
            }

            public static double GetNumber(string featureName)
            {
                ThrowIfMagifyIsNotReady(nameof(GetNumber));
                return MagifyPlatformAPI.Features.GetNumber(featureName);
            }

            public static bool TryGetString(string featureName, out string result)
            {
                ThrowIfMagifyIsNotReady(nameof(TryGetString));
                return MagifyPlatformAPI.Features.TryGetString(featureName, out result);
            }

            public static string GetString(string featureName)
            {
                ThrowIfMagifyIsNotReady(nameof(GetString));
                return MagifyPlatformAPI.Features.GetString(featureName);
            }
        }
    }
}