using System.Collections.Generic;
using System.IO;
using Magify.Model;

namespace Magify
{
    internal class AnalyticsEventStorage<T>
    {
        public const string AnalyticsFolderName = "analytics";

        private readonly string _fileName;
        private readonly FileStorage _storage;

        public AnalyticsEventStorage(EventType eventType, string storagePath)
        {
            _fileName = $"{eventType}.json";
            _storage = new FileStorage(AnalyticsFolderName, Path.Combine(storagePath));
        }

        public bool Exists()
        {
            return _storage.Exists(_fileName);
        }

        public List<T> LoadEvents()
        {
            var serializedEvents = _storage.Load(_fileName);
            return JsonFacade.DeserializeObject<List<T>>(serializedEvents);
        }

        public void SaveEvents(IReadOnlyCollection<T> events)
        {
            var serializedEvents = JsonFacade.SerializeObject(events);
            _storage.Save(_fileName, serializedEvents);
        }

        public void Clear()
        {
            _storage.Clear(_fileName);
        }
    }
}