﻿using System;
using System.Threading;
using System.Threading.Tasks;
using JetBrains.Annotations;

namespace Magify
{
    internal class ServerRequestTools<T> : IDisposable
    {
        [NotNull]
        private CancellationTokenSource _allRequestsToken = new();
        [CanBeNull]
        private TaskCompletionSource<T> _completionSource;
        [NotNull]
        public ReusableCancellationTokenSource RequestRepeatingDelayToken { get; } = new();
        [NotNull]
        public ExponentialBackoff Backoff { get; } = new();
        [CanBeNull]
        public Task<T> CompletionTask => _completionSource?.Task;
        public CancellationToken AllRequestsToken => _allRequestsToken.Token;

        public bool TryCreateNewCompletionSource([NotNull] out Task<T> completionTask)
        {
            if (_completionSource == null)
            {
                _completionSource = new TaskCompletionSource<T>();
                completionTask = _completionSource.Task!;
                return true;
            }

            completionTask = _completionSource.Task!;
            return false;
        }

        public void TrySetResult([CanBeNull] T result)
        {
            _completionSource?.TrySetResult(result);
            _completionSource = null;
        }

        public void CancelAllRequests()
        {
            _allRequestsToken.Cancel();
            _allRequestsToken.Dispose();
            _allRequestsToken = new CancellationTokenSource();
        }

        public void ResetRequestsRepeatDelay()
        {
            RequestRepeatingDelayToken.CancelCurrent();
        }

        public void Dispose()
        {
            _allRequestsToken.Cancel();
            _allRequestsToken.Dispose();
            RequestRepeatingDelayToken.CancelCurrent();
            _completionSource?.TrySetCanceled();
            _completionSource = null;
        }
    }
}