using System.IO;

namespace Magify
{
    internal class FileStorage
    {
        private const string DefaultFolderName = "files";
        public string RootFolderPath { get; }

        public FileStorage(string storagePath) : this(DefaultFolderName, storagePath)
        {
        }

        public FileStorage(string rootFolderName, string storagePath)
        {
            RootFolderPath = Path.Combine(storagePath, rootFolderName);
        }

        public void Save(string fileName, string data)
        {
            File.WriteAllText(GetFilePath(fileName, true), data);
        }

        public string Load(string fileName)
        {
            var filePath = GetFilePath(fileName, false);
            return !File.Exists(filePath) ? string.Empty : File.ReadAllText(filePath);
        }

        public void Clear(string fileName)
        {
            var filePath = GetFilePath(fileName, false);
            if (Exists(filePath))
            {
                File.Delete(filePath);
            }
        }

        /// <param name="sourceFilePath">Must exist</param>
        public void CopyTextFrom(string sourceFilePath, string destinationFileName)
        {
            var destinationFilePath = GetFilePath(destinationFileName, true);
            File.Copy(sourceFilePath, destinationFilePath, true);
        }

        internal bool Exists(string fileName)
        {
            return File.Exists(GetFilePath(fileName, false));
        }

        internal string GetFilePath(string fileName, bool createDirectory)
        {
            if (createDirectory)
            {
                Directory.CreateDirectory(RootFolderPath);
            }
            return Path.Combine(RootFolderPath, fileName);
        }
    }
}